package jmodeler;

import jgl.GLU;

public class Camera {
  Point3d viewPoint = new Point3d(0.0,0.0,0.0);
  Point3d cameraPosition = new Point3d(0.0,0.0,2.0);
  Point3d upVector = new Point3d(0.0,1.0,0.0);

  public Camera() {
  }

  public Point3d getCameraPosition() {
    return cameraPosition;
  }

  public void setCameraPosition(double x,double y,double z) {
    cameraPosition.x = x;
    cameraPosition.y = y;
    cameraPosition.z = z;
  }
  public void setUpVector(double x,double y,double z) {
    upVector.x = x;
    upVector.y = y;
    upVector.z = z;
  }
  public Point3d getUpVector() {
    return upVector;
  }
  public Point3d getViewPoint() {
    return viewPoint;
  }
  public void setViewPoint(double x,double y,double z) {
    viewPoint.x = x;
    viewPoint.y = y;
    viewPoint.z = z;
  }

  public void rotateUpVector(double angle){
    double costheta = Math.cos(angle);
    double sintheta = Math.sin(angle);
    Point3d result = new Point3d(0,0,0);
    Point3d viewVector = new Point3d(0,0,0);
    viewVector.x = viewPoint.x - cameraPosition.x;
    viewVector.y = viewPoint.y - cameraPosition.y;
    viewVector.z = viewPoint.z - cameraPosition.z;

    Point3d r = viewVector.normalize();
    result.x += (costheta + (1 - costheta) * r.x * r.x) * upVector.x;
    result.x += ((1 - costheta) * r.x * r.y - r.z * sintheta) * upVector.y;
    result.x += ((1 - costheta) * r.x * r.z + r.y * sintheta) * upVector.z;

    result.y += ((1 - costheta) * r.x * r.y + r.z * sintheta) * upVector.x;
    result.y += (costheta + (1 - costheta) * r.y * r.y) * upVector.y;
    result.y += ((1 - costheta) * r.y * r.z - r.x * sintheta) * upVector.z;

    result.z += ((1 - costheta) * r.x * r.z - r.y * sintheta) * upVector.x;
    result.z += ((1 - costheta) * r.y * r.z + r.x * sintheta) * upVector.y;
    result.z += (costheta + (1 - costheta) * r.z * r.z) * upVector.z;

    upVector.x = result.x;
    upVector.y = result.y;
    upVector.z = result.z;
  }

  public void rotateViewVector(double angle){
    double costheta = Math.cos(angle);
    double sintheta = Math.sin(angle);
    Point3d result = new Point3d(0,0,0);
    Point3d viewVector = new Point3d(0,0,0);
    viewVector.x = - viewPoint.x + cameraPosition.x;
    viewVector.y = - viewPoint.y + cameraPosition.y;
    viewVector.z = - viewPoint.z + cameraPosition.z;

    Point3d r = upVector.normalize();
    result.x += (costheta + (1 - costheta) * r.x * r.x) * viewVector.x;
    result.x += ((1 - costheta) * r.x * r.y - r.z * sintheta) * viewVector.y;
    result.x += ((1 - costheta) * r.x * r.z + r.y * sintheta) * viewVector.z;

    result.y += ((1 - costheta) * r.x * r.y + r.z * sintheta) * viewVector.x;
    result.y += (costheta + (1 - costheta) * r.y * r.y) * viewVector.y;
    result.y += ((1 - costheta) * r.y * r.z - r.x * sintheta) * viewVector.z;

    result.z += ((1 - costheta) * r.x * r.z - r.y * sintheta) * viewVector.x;
    result.z += ((1 - costheta) * r.y * r.z + r.x * sintheta) * viewVector.y;
    result.z += (costheta + (1 - costheta) * r.z * r.z) * viewVector.z;

    cameraPosition.x = result.x + viewPoint.x;
    cameraPosition.y = result.y + viewPoint.y;
    cameraPosition.z = result.z + viewPoint.z;
  }

  public void rotate(double angleUp, double angleView){
    rotateUpVector(angleUp);
    rotateViewVector(angleView);
  }

  public void rotateView(double angle,double x,double y,double z)
  {
    Point3d newViewPosition = new Point3d(0,0,0);

    // Get the view vector (The direction we are facing)
    Point3d viewVector = new Point3d(0,0,0);
    viewVector.x = viewPoint.x - cameraPosition.x;
    viewVector.y = viewPoint.y - cameraPosition.y;
    viewVector.z = viewPoint.z - cameraPosition.z;

    // Calculate the sine and cosine of the angle once
    double cosTheta = Math.cos(angle);
    double sinTheta = Math.sin(angle);

    // Find the new x position for the new rotated point
    newViewPosition.x = (cosTheta + (1 - cosTheta) * x * x) * viewVector.x;
    newViewPosition.x += ( (1 - cosTheta) * x * y - z * sinTheta) * viewVector.y;
    newViewPosition.x += ( (1 - cosTheta) * x * z + y * sinTheta) * viewVector.z;

    // Find the new y position for the new rotated point
    newViewPosition.y = ( (1 - cosTheta) * x * y + z * sinTheta) * viewVector.x;
    newViewPosition.y += (cosTheta + (1 - cosTheta) * y * y) * viewVector.y;
    newViewPosition.y += ( (1 - cosTheta) * y * z - x * sinTheta) * viewVector.z;

    // Find the new z position for the new rotated point
    newViewPosition.z = ( (1 - cosTheta) * x * z - y * sinTheta) * viewVector.x;
    newViewPosition.z += ( (1 - cosTheta) * y * z + x * sinTheta) * viewVector.y;
    newViewPosition.z += (cosTheta + (1 - cosTheta) * z * z) * viewVector.z;

    // Now we just add the newly rotated vector to our position to set
    // our new rotated view of our camera.
    viewPoint.x = cameraPosition.x + newViewPosition.x;
    viewPoint.y = cameraPosition.y + newViewPosition.y;
    viewPoint.z = cameraPosition.z + newViewPosition.z;
  }

  public void gluLookAt(GLU glu){
    glu.gluLookAt(cameraPosition.x,cameraPosition.y,cameraPosition.z,
                  viewPoint.x,viewPoint.y,viewPoint.z,
                  upVector.x,upVector.y,upVector.z);
  }
}
