package jmodeler;

import javax.swing.*;
import java.awt.*;
import com.borland.jbcl.layout.*;
import java.awt.event.*;
import java.io.File;

/**
 * <p>Title: jModeler</p>
 * <p>Description: Create and View Bezier curves and surfaces</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Computer Aided Graphics Design</p>
 * @author (biswarup, chiragp, devesh, dharm, vikasj, jikku)@cse.iitb.ac.in
 * @version 1.0
 */

public class CreateCurveFrame
    extends JFrame {
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanel1 = new JPanel();
  JPanel jPanel2 = new JPanel();
  JButton cancelButton = new JButton();
  JButton okButton = new JButton();
  JPanel jPanel3 = new JPanel();
  JTextField fileTextField = new JTextField();
  JButton browseButton = new JButton();
  VerticalFlowLayout verticalFlowLayout1 = new VerticalFlowLayout();
  JPanel jPanel4 = new JPanel();
  JSpinner segLenSpinner = new JSpinner();
  JLabel jLabel1 = new JLabel();
  JLabel jLabel2 = new JLabel();
  JButton colorChooserButton = new JButton();
  Color curveColor = Color.BLACK;
  SpinnerModel sModel = new SpinnerNumberModel(0.10,0.00,1.00,0.05);
  jModelManager manager  = null;
  JPanel jPanel5 = new JPanel();
  JLabel jLabel3 = new JLabel();
  JTextField curveNameTextField = new JTextField();
  JLabel jLabel4 = new JLabel();

  public CreateCurveFrame(jModelManager m) {
    try {
      jbInit();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
    this.manager = m;
    colorChooserButton.setBackground(curveColor);
  }

  public CreateCurveFrame(jModelManager m,String name) {
    this(m);
    curveNameTextField.setText(name);
  }

  public void setCurveName(String name){
    curveNameTextField.setText(name);
  }

  void jbInit() throws Exception {
    this.getContentPane().setLayout(borderLayout1);
    cancelButton.setText("Cancel");
    cancelButton.addActionListener(new CreateCurveFrame_cancelButton_actionAdapter(this));
    okButton.setText("Ok");
    okButton.addActionListener(new CreateCurveFrame_okButton_actionAdapter(this));
    jPanel2.setLayout(verticalFlowLayout1);
    fileTextField.setMinimumSize(new Dimension(4, 19));
    fileTextField.setPreferredSize(new Dimension(150, 19));
    fileTextField.setText("");
    browseButton.setIcon(new ImageIcon(CreateCurveFrame.class.getResource(
        "folder19x19.png")));
    browseButton.setMargin(new Insets(1, 1, 1, 1));
    browseButton.setMnemonic('0');
    browseButton.setText("");
    browseButton.addActionListener(new
                                   CreateCurveFrame_browseButton_actionAdapter(this));
    segLenSpinner.setModel(sModel);
    segLenSpinner.setPreferredSize(new Dimension(50, 20));
    segLenSpinner.setVerifyInputWhenFocusTarget(true);
    jLabel1.setText("Segment Length");
    jLabel2.setText("File :");
    colorChooserButton.setPreferredSize(new Dimension(27, 24));
    colorChooserButton.setIcon(null);
    colorChooserButton.setMargin(new Insets(1, 1, 1, 1));
    colorChooserButton.setText("");
    colorChooserButton.addActionListener(new
        CreateCurveFrame_colorChooserButton_actionAdapter(this));
    jLabel3.setText("Name :");
    curveNameTextField.setPreferredSize(new Dimension(70, 19));
    curveNameTextField.setText("");
    jLabel4.setText("Color :");
    this.getContentPane().add(jPanel1, BorderLayout.SOUTH);
    jPanel1.add(okButton, null);
    jPanel1.add(cancelButton, null);
    this.getContentPane().add(jPanel2, BorderLayout.CENTER);
    jPanel2.add(jPanel5, null);
    jPanel5.add(jLabel3, null);
    jPanel5.add(curveNameTextField, null);
    jPanel5.add(jLabel4, null);
    jPanel5.add(colorChooserButton, null);
    jPanel2.add(jPanel3, null);
    jPanel3.add(jLabel2, null);
    jPanel3.add(fileTextField, null);
    jPanel3.add(browseButton, null);
    jPanel2.add(jPanel4, null);
    jPanel4.add(jLabel1, null);
    jPanel4.add(segLenSpinner, null);
    this.setTitle("Curve Create");
  }

  void browseButton_actionPerformed(ActionEvent e) {
    JFileChooser fc = new JFileChooser();
    int retVal = fc.showOpenDialog(this);
    if (retVal == JFileChooser.APPROVE_OPTION) {
      File f = fc.getSelectedFile();
      fileTextField.setText(f.getAbsolutePath());
    }
  }

  void colorChooserButton_actionPerformed(ActionEvent e) {
    Color newColor = JColorChooser.showDialog(
        this,
        "Choose Color of the Curve",
        this.getBackground());
    if( newColor == null ) return;
    curveColor = newColor;
    colorChooserButton.setBackground(curveColor);
  }

  void okButton_actionPerformed(ActionEvent e) {

    if( fileTextField.getText().trim().length() == 0 ){
      JOptionPane.showMessageDialog(this,"You need to choose a file first","Error : No File Selected",JOptionPane.ERROR_MESSAGE);
      return;
    }
    if( manager.getObjectListModel().contains(curveNameTextField.getText()) ){
      JOptionPane.showMessageDialog(this,"You need to choose a unique name for the curve","Error : Duplicate Curve Name",JOptionPane.ERROR_MESSAGE);
      return;
    }
    Curve curve = null;
    try{
      curve = new Curve(curveNameTextField.getText(),
                              fileTextField.getText(), curveColor,
                              ( (Double) segLenSpinner.getValue()).doubleValue());
    }catch(Exception ex){
      JOptionPane.showMessageDialog(this,"Could not create curve because of :\n"+ex.toString(),"Error : "+ex.toString(),JOptionPane.ERROR_MESSAGE);
      return;
    }
    manager.addCurve(curve);
    this.setVisible(false);
  }

  void cancelButton_actionPerformed(ActionEvent e) {
    this.setVisible(false);
  }
}

class CreateCurveFrame_browseButton_actionAdapter
    implements java.awt.event.ActionListener {
  CreateCurveFrame adaptee;

  CreateCurveFrame_browseButton_actionAdapter(CreateCurveFrame adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.browseButton_actionPerformed(e);
  }
}

class CreateCurveFrame_colorChooserButton_actionAdapter
    implements java.awt.event.ActionListener {
  CreateCurveFrame adaptee;

  CreateCurveFrame_colorChooserButton_actionAdapter(CreateCurveFrame adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.colorChooserButton_actionPerformed(e);
  }
}

class CreateCurveFrame_okButton_actionAdapter implements java.awt.event.ActionListener {
  CreateCurveFrame adaptee;

  CreateCurveFrame_okButton_actionAdapter(CreateCurveFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.okButton_actionPerformed(e);
  }
}

class CreateCurveFrame_cancelButton_actionAdapter implements java.awt.event.ActionListener {
  CreateCurveFrame adaptee;

  CreateCurveFrame_cancelButton_actionAdapter(CreateCurveFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.cancelButton_actionPerformed(e);
  }
}
