package jmodeler;

import javax.swing.*;
import java.awt.*;
import com.borland.jbcl.layout.*;
import java.awt.event.*;
import java.io.File;

/**
 * <p>Title: jModeler</p>
 * <p>Description: Create and View Besier curves and surfaces</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Computer Aided Graphics Design</p>
 * @author (biswarup, chiragp, devesh, dharm, vikasj, jikku)@cse.iitb.ac.in
 * @version 1.0
 */

public class CreateSurfaceFrame extends JFrame {
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel jPanel1 = new JPanel();
  JPanel jPanel2 = new JPanel();
  JButton jButton1 = new JButton();
  JButton jButton2 = new JButton();
  VerticalFlowLayout verticalFlowLayout1 = new VerticalFlowLayout();
  JPanel jPanel3 = new JPanel();
  JLabel jLabel1 = new JLabel();
  JTextField fileTextField = new JTextField();
  JButton browseButton = new JButton();
  JButton colorChooserButton = new JButton();
  JPanel jPanel4 = new JPanel();
  JLabel jLabel2 = new JLabel();
  JSpinner uSegLenSpinner = new JSpinner();
  JLabel jLabel3 = new JLabel();
  JSpinner vSegLenSpinner = new JSpinner();
  Color surfaceColor = Color.BLACK;
  jModelManager manager = null;
  SpinnerModel vSegLenModel = new SpinnerNumberModel(0.10,0.00,1.00,0.05);
  SpinnerModel uSegLenModel = new SpinnerNumberModel(0.10,0.00,1.00,0.05);
  JPanel jPanel5 = new JPanel();
  JLabel jLabel4 = new JLabel();
  JTextField surfaceNameTextField = new JTextField();
  JLabel jLabel5 = new JLabel();

  public CreateSurfaceFrame(jModelManager m) {
    try {
      jbInit();
    }
    catch(Exception ex) {
      ex.printStackTrace();
    }
    this.manager = m;
    colorChooserButton.setBackground(surfaceColor);
  }

  public CreateSurfaceFrame(jModelManager m,String name){
    this(m);
    surfaceNameTextField.setText(name);
  }

  public void setSurfaceName(String name){
    surfaceNameTextField.setText(name);
  }

  void jbInit() throws Exception {
    this.getContentPane().setLayout(borderLayout1);
    jButton1.setText("Cancel");
    jButton1.addActionListener(new CreateSurfaceFrame_jButton1_actionAdapter(this));
    jButton2.setText("Ok");
    jButton2.addActionListener(new CreateSurfaceFrame_jButton2_actionAdapter(this));
    jPanel2.setLayout(verticalFlowLayout1);
    jLabel1.setText("File :");
    fileTextField.setPreferredSize(new Dimension(150, 19));
    fileTextField.setText("");
    browseButton.setIcon(new ImageIcon(CreateSurfaceFrame.class.getResource("folder19x19.png")));
    browseButton.setMargin(new Insets(1, 1, 1, 1));
    browseButton.setText("");
    browseButton.addActionListener(new CreateSurfaceFrame_browseButton_actionAdapter(this));
    colorChooserButton.setPreferredSize(new Dimension(27, 24));
    colorChooserButton.setText("");
    colorChooserButton.addActionListener(new CreateSurfaceFrame_colorChooserButton_actionAdapter(this));
    jLabel2.setRequestFocusEnabled(true);
    jLabel2.setText("u-segLen :");
    jLabel3.setText("v-segLen :");
    uSegLenSpinner.setModel(uSegLenModel);
    uSegLenSpinner.setPreferredSize(new Dimension(50, 20));
    vSegLenSpinner.setModel(vSegLenModel);
    vSegLenSpinner.setMinimumSize(new Dimension(28, 20));
    vSegLenSpinner.setPreferredSize(new Dimension(50, 20));
    jLabel4.setText("Name :");
    surfaceNameTextField.setPreferredSize(new Dimension(70, 19));
    surfaceNameTextField.setText("");
    jLabel5.setText("Color :");
    this.getContentPane().add(jPanel1,  BorderLayout.SOUTH);
    jPanel1.add(jButton2, null);
    jPanel1.add(jButton1, null);
    this.getContentPane().add(jPanel2, BorderLayout.CENTER);
    jPanel2.add(jPanel5, null);
    jPanel5.add(jLabel4, null);
    jPanel5.add(surfaceNameTextField, null);
    jPanel5.add(jLabel5, null);
    jPanel5.add(colorChooserButton, null);
    jPanel2.add(jPanel3, null);
    jPanel3.add(jLabel1, null);
    jPanel3.add(fileTextField, null);
    jPanel3.add(browseButton, null);
    jPanel2.add(jPanel4, null);
    jPanel4.add(jLabel2, null);
    jPanel4.add(uSegLenSpinner, null);
    jPanel4.add(jLabel3, null);
    jPanel4.add(vSegLenSpinner, null);
  }

  void browseButton_actionPerformed(ActionEvent e) {
    JFileChooser fc = new JFileChooser();
    int retVal = fc.showOpenDialog(this);
    if (retVal == JFileChooser.APPROVE_OPTION) {
      File f = fc.getSelectedFile();
      fileTextField.setText(f.getAbsolutePath());
    }
  }

  void colorChooserButton_actionPerformed(ActionEvent e) {
    Color newColor = JColorChooser.showDialog(
        this,
        "Choose Color of the Curve",
        this.getBackground());
    if( newColor == null ) return;
    surfaceColor = newColor;
    colorChooserButton.setBackground(surfaceColor);
  }

  void jButton2_actionPerformed(ActionEvent e) {
    if( fileTextField.getText().trim().length() == 0 ){
      JOptionPane.showMessageDialog(this,"You need to choose a file first","Error : No File Selected",JOptionPane.ERROR_MESSAGE);
      return;
    }
    if( manager.getObjectListModel().contains(surfaceNameTextField.getText()) ){
      JOptionPane.showMessageDialog(this,"You need to choose a unique name for the curve","Error : Duplicate Curve Name",JOptionPane.ERROR_MESSAGE);
      return;
    }
    Surface surface = null;
    try{
      surface = new Surface(surfaceNameTextField.getText(),
                            fileTextField.getText(), surfaceColor,
                            ( (Double) uSegLenSpinner.getValue()).doubleValue());
    }catch(Exception ex){
      JOptionPane.showMessageDialog(this,"Could not create surface because of :\n"+ex.toString(),"Error : "+ex.toString(),JOptionPane.ERROR_MESSAGE);
      return;
    }
    manager.addSurface(surface);
    this.setVisible(false);
  }

  void jButton1_actionPerformed(ActionEvent e) {
    this.setVisible(false);
  }
}

class CreateSurfaceFrame_browseButton_actionAdapter implements java.awt.event.ActionListener {
  CreateSurfaceFrame adaptee;

  CreateSurfaceFrame_browseButton_actionAdapter(CreateSurfaceFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.browseButton_actionPerformed(e);
  }
}

class CreateSurfaceFrame_colorChooserButton_actionAdapter implements java.awt.event.ActionListener {
  CreateSurfaceFrame adaptee;

  CreateSurfaceFrame_colorChooserButton_actionAdapter(CreateSurfaceFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.colorChooserButton_actionPerformed(e);
  }
}

class CreateSurfaceFrame_jButton2_actionAdapter implements java.awt.event.ActionListener {
  CreateSurfaceFrame adaptee;

  CreateSurfaceFrame_jButton2_actionAdapter(CreateSurfaceFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jButton2_actionPerformed(e);
  }
}

class CreateSurfaceFrame_jButton1_actionAdapter implements java.awt.event.ActionListener {
  CreateSurfaceFrame adaptee;

  CreateSurfaceFrame_jButton1_actionAdapter(CreateSurfaceFrame adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.jButton1_actionPerformed(e);
  }
}
