package jmodeler;

import java.io.*;
import java.util.*;
import java.awt.*;

/**
 * <p>Title: jModeler</p>
 * <p>Description: Create and View Bezier curves and surfaces</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Computer Aided Graphics Design</p>
 * @author (biswarup, chiragp, devesh, dharm, vikasj, jikku)@cse.iitb.ac.in
 * @version 1.0
 */


public class Curve { //extends Object {
  //control Points
  Vector ctrPt = new Vector(0);
  double segLen;
  int finished = 0; //finished == 1 means Curve formation complete
  boolean visible = true;
  boolean showControlPts = false;
  boolean drawTangent = false;
  double t =0;
  String name = null;
  Point3d p = null; //a point to be shown by the curvature, tangent etc.
  boolean showp = false;

  java.awt.Color color;
  private boolean removed = false;


  public void sett(double t){
    this.t = t;
  }
  public Point3d return_tangent(double t){
          int num_ctr_pts = ctrPt.size();
          Point3d to_be_ret = new Point3d(0,0,0);
          for(int i=0;i<num_ctr_pts;i++){
              double diff1 = diff(num_ctr_pts,i,t);
              to_be_ret.x += ((Point3d) ctrPt.elementAt(i)).x * diff1;
              to_be_ret.y += ((Point3d) ctrPt.elementAt(i)).y * diff1;
              to_be_ret.z += ((Point3d) ctrPt.elementAt(i)).z * diff1;
          }

          double denom = Math.sqrt(Math.pow(to_be_ret.x,2) + Math.pow(to_be_ret.y,2) + Math.pow(to_be_ret.z,2)); //denominator for normalizing.
          to_be_ret.x /= denom; to_be_ret.y /= denom; to_be_ret.z /= denom;

          return to_be_ret;
      }

      public double diff(int num,int i2,double t){
          return (num * (bernstein_val(num-1,i2-1,t) - bernstein_val(num-1,i2,t)));
      }


      public double bernstein_val(int n1,int i1,double t){
          return ( factorial(n1) /( factorial (i1) * factorial(n1-i1)) * Math.pow(t,i1) * Math.pow(1-t,n1-i1));
      }


  public Curve(double[][] points,Color _color,double _segLen){
    for(int i=0;i<points.length;i++){
      Point3d p = new Point3d(points[i][0],points[i][2],points[i][2]);
      ctrPt.add(p);
    }
    segLen = _segLen;
    color = _color;
    finished = 1;
  }

  public Curve(String _name,String file, java.awt.Color _color, double _segLen) throws FileNotFoundException,IOException{ //input will be file to parse
    //control points created

    java.io.BufferedReader reader = null;
    reader = new java.io.BufferedReader(new FileReader(file));


    //String  s = "";
    String line = "";
    String token = "";
    int m = 0, old_m = 0;
    int n = 0;

    while ( (line = reader.readLine()) != null) {
      n++;
      StringTokenizer st = new StringTokenizer(line, " ");
      if (n == 1) {
        old_m = m;
      }
      else {
        //if(old_m != m){
        //System.err.println("File Format BAD: No. of coloums not same");
        //System.err.println("File Format BAD: No. of rows != 1");
        throw new FileNotFoundException("File Format BAD: No. of rows != 1");
      }

      m = 0;
      while ( st.hasMoreTokens() ) {
        double x = Double.parseDouble(st.nextToken());
        double y = Double.parseDouble(st.nextToken());
        double z = Double.parseDouble(st.nextToken());
        Point3d p = new Point3d(x, y, z);
        ctrPt.add(new Point3d(x, y, z));
        m++;
      }
    }

    reader.close();

    //color
    color = _color;
    //segment length
    segLen = _segLen;
    name = _name;
    finished = 1;
    return;
  }

  public boolean equals(Object o){
    String s = o.toString();
    return s.equals(name);
  }

  public String toString(){
    return new String(name);
  }

  public boolean isFormed() {
    if (this.finished == 1) {
      return true;
    }
    else {
      return false;
    }
  }

  public Color getColor() {
    return this.color;
  }

  public double getSeglength() {
    return this.segLen;
  }

  public Point3d[] getControlPts() {
    return (Point3d[]) ctrPt.toArray();
  }

  public long factorial(long x) {
    int output = 1;
    for (int i = 1; i <= x; i++) {
      output = output * i;
    }
    return output;
    /*if(x < 0) { throw new IllegalArgumentException("Argument less than zero"); }
       return (x <= 1) ? 1 : (x * factorial(x-1));
     */
  }

  public Point3d calculateAt(double u){
      double au = 1- u;
      int o = ctrPt.size();
      double[] A = new double[o];
      double tempx=0.0f, tempy=0.0f, tempz=0.0f;
      Object[] ptArray = ctrPt.toArray();
      Point3d[] ctrPtArray = new Point3d[ptArray.length+1];
      for (int i = 0; i < ptArray.length; i++) {
        ctrPtArray[i] = (Point3d) ptArray[i];
      }

      for (int k = 0; k < o; k++) {
              //System.out.println(nctrlP[k].x+" "+nctrlP[k].y+" "+nctrlP[k].z);
              A[k] = (factorial( (long) (o-1)) /
                      (factorial( (long) k) * factorial( (long) (o -1- k)))) *
                  Math.pow(u, k) * Math.pow(au, (o - 1 - k));

              tempx = tempx + (ctrPtArray[k].x) * A[k];
              tempy = tempy + (ctrPtArray[k].y) * A[k];
              tempz = tempz + (ctrPtArray[k].z) * A[k];
            }
            return ( new Point3d(tempx,tempy,tempz));

    }

  public void makep(double u){
    double au = 1- u;
    int o = ctrPt.size();
    double[] A = new double[o];
    double tempx=0.0f, tempy=0.0f, tempz=0.0f;
    Object[] ptArray = ctrPt.toArray();
    Point3d[] ctrPtArray = new Point3d[ptArray.length+1];
    for (int i = 0; i < ptArray.length; i++) {
      ctrPtArray[i] = (Point3d) ptArray[i];
    }

    for (int k = 0; k < o; k++) {
            //System.out.println(nctrlP[k].x+" "+nctrlP[k].y+" "+nctrlP[k].z);
            A[k] = (factorial( (long) (o-1)) /
                    (factorial( (long) k) * factorial( (long) (o -1- k)))) *
                Math.pow(u, k) * Math.pow(au, (o - 1 - k));

            tempx = tempx + (ctrPtArray[k].x) * A[k];
            tempy = tempy + (ctrPtArray[k].y) * A[k];
            tempz = tempz + (ctrPtArray[k].z) * A[k];
          }
          this.p = new Point3d(tempx,tempy,tempz);

  }
  public Point3d[] generateRenderingPts() {
    //increment and  make the line segments and return them
    double itr = (double) (1.0 / segLen);

    int o = ctrPt.size();
    double u = 0;
    double[] A = new double[o];
    Point3d[] output = new Point3d[ (int) (itr+1)];
    Object[] ptArray = ctrPt.toArray();
    Point3d[] ctrPtArray = new Point3d[ptArray.length+1];
    for (int i = 0; i < ptArray.length; i++) {
      ctrPtArray[i] = (Point3d) ptArray[i];
    }
    for (int i = 0; i <=itr; i++) {
      u = i * segLen;
      double au = 1. - u;
      double tempx = 0;
      double tempy = 0;
      double tempz = 0;

      for (int k = 0; k < o; k++) {
        //System.out.println(nctrlP[k].x+" "+nctrlP[k].y+" "+nctrlP[k].z);
        A[k] = (factorial( (long) (o-1)) /
                (factorial( (long) k) * factorial( (long) (o -1- k)))) *
            Math.pow(u, k) * Math.pow(au, (o - 1 - k));

        tempx = tempx + (ctrPtArray[k].x) * A[k];
        tempy = tempy + (ctrPtArray[k].y) * A[k];
        tempz = tempz + (ctrPtArray[k].z) * A[k];
      }

      output[i] = new Point3d(tempx, tempy, tempz);
      //System.out.println(finalP[i].x+" "+finalP[i].y+" "+finalP[i].z);
    } ///end loop iter (i)
    Point3d p = (Point3d) ctrPt.lastElement();
    output[output.length-1]=new Point3d(p.x,p.y,p.z);
    return output;
  }
  public boolean isVisible() {
    return visible;
  }
  public void setVisible(boolean visible) {
    this.visible = visible;
  }
  public String getName() {
    return name;
  }
  public void setName(String name) {
    this.name = name;
  }

  /**
   * setRemoved
   */
  public void setRemoved() {
    removed = true;
  }

  public boolean isRemoved() {
    return removed;
  }

  public void toggleshowControlPts(){
    showControlPts = !showControlPts;
  }

}
