package jmodeler;

import jgl.GL;
import jgl.GLUT;
import jgl.GLCanvas;
import java.util.Vector;
import java.awt.Color;

/**
 * <p>Title: jModeler</p>
 * <p>Description: Create and View Bezier curves and surfaces</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: Computer Aided Graphics Design</p>
 * @author (biswarup, chiragp, devesh, dharm, vikasj, jikku)@cse.iitb.ac.in
 * @version 1.0
 */

public class Viewer
    extends GLCanvas {

  double rotateAngleX = 0, rotateAngleY = 0;
  double transX = 0, transY = 0;
  double range = 12.0f, rangeMin = 1.0f, rangeMax = 40.0f, zoomStepMultiplier = 0.1f, zoomStep=0.3f;
  int curX, curY;
  int width, height;
  boolean rotateState = false;
  boolean translateState = false;
  Vector curves = new Vector();
  Vector surfaces = new Vector();
  boolean axisVisible = false;
  int winWidth = 640, winHeight = 480;
  double rotateStep = 0.1;

  //float no_mat[] = {
    //  0.0f, 0.0f, 0.0f, 1.0f};
  //float mat_ambient[] = {
    //  0.7f, 0.7f, 0.7f, 1.0f};
  //float mat_ambient_color[] = {
   //   0.8f, 0.8f, 0.2f, 1.0f};
  //float mat_diffuse[] = {
  //    0.1f, 0.5f, 0.8f, 1.0f};
  //float mat_specular[] = {
   //   0.0f, 0.0f, 0.0f, 1.0f};
  //float no_shininess[] = {
   //   0.0f};
  //float low_shininess[] = {
  //    0.0f};
  //float high_shininess[] = {
  //    100.0f};
  //float mat_emission[] = {
  //    0.3f, 0.2f, 0.2f, 0.0f};
  float[] lightAmbient = {
      1.0f, 1.0f, 1.0f, 1.0f};
  float[] lightDiffuse = {
      1.0f, 1.0f, 1.0f, 1.0f};
  float[] lightSpecular = {
      0.0f, 0.0f, 0.0f, 1.0f};

  float[] globalAmbient = {
      0.5f, 0.5f, 0.5f, 1.0f};
  float[] light0Position = {
      0.7f, 0.7f, 0.7f, 0.0f};
  float[] light1Position = {
      0.7f, 0.7f, -0.7f, 0.0f};
  public void addCurve(Curve c) {
    curves.add(c);
    try {
      myUT.glutPostRedisplay();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

public void setMouseSense(double z){
    rotateStep = z;
  }
  public void setZoomStep(double z){
    zoomStep = z;
    if (z > (rangeMax - rangeMin)){
      System.err.println("zoom step cannot be > difference of max and min range");
      z = rangeMax - rangeMin;
    }
    System.out.println("++"+zoomStep);
  }

  public void setRangeMax(double z){
    rangeMax = z;
    if(rangeMax < rangeMin){
      System.err.println("Max range cannot be less than min range");
      rangeMax = rangeMin;
    }
  }

  public void setRangeMin(double z){
    rangeMin = z;
    if(rangeMin > rangeMax){
      System.err.println("Min range cannot be greater than max range");
      rangeMin = rangeMax;
    }
  }

  public void addSurface(Surface s) {
    surfaces.add(s);
    try {
      myUT.glutPostRedisplay();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void zoomIn() {
    //zoomStep = (rangeMax - range) * zoomStepMultiplier;
    double nrange = range - zoomStep;
    if (nrange < rangeMin) {
      range = rangeMin;
    }
    else if (nrange > rangeMax) {
      range = rangeMax;
    }
    else {
      range = nrange;
    }
    myReshape(winWidth, winHeight);
    myUT.glutPostRedisplay();
  }

  public void zoomOut() {
    //zoomStep = (rangeMax - range) * zoomStepMultiplier;
    double nrange = range + zoomStep;
    if (nrange < rangeMin) {
      range = rangeMin;
    }
    else if (nrange > rangeMax) {
      range = rangeMax;
    }
    else {
      range = nrange;
    }
    myReshape(winWidth, winHeight);
    myUT.glutPostRedisplay();
  }

  public void zoomReset() {
    range = 12.0f;
    myReshape(winWidth, winHeight);
    myUT.glutPostRedisplay();
  }

  public Viewer() {
    myUT.glutInitWindowSize(winWidth, winHeight);
    myUT.glutInitWindowPosition(0, 0);
    myUT.glutCreateWindow(this);
    myInit();
    myUT.glutReshapeFunc("myReshape");
    myUT.glutDisplayFunc("display");
    myUT.glutKeyboardFunc("keyboard");
    myUT.glutMouseFunc("mouse");
    myUT.glutMotionFunc("mouseMotion");
    float[] lightAmbient = {
        0.5f, 0.5f, 0.5f, 1.0f};
    myGL.glEnable(myGL.GL_LIGHTING);

    myUT.glutMainLoop();
  }

  public void setAxisVisible(boolean b) {
    axisVisible = b;
    myUT.glutPostRedisplay();
  }

  public boolean getAxisVisible() {
    return axisVisible;
  }

  public void showAxis() {
    setAxisVisible(true);
  }

  public void hideAxis() {
    setAxisVisible(false);
  }

  public void display() {
    try {
      //myGL.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
      myGL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
      myGL.glPushMatrix();
      myGL.glEnable(GL.GL_DEPTH_TEST);
      myGL.glShadeModel(GL.GL_SMOOTH);
      myGL.glEnable(GL.GL_NORMALIZE);
      if( cullface ){
        myGL.glEnable(GL.GL_CULL_FACE);
      }else{
        myGL.glDisable(GL.GL_CULL_FACE);
      }

      myGL.glEnable(GL.GL_LIGHTING);
      myGL.glLightModelfv(GL.GL_LIGHT_MODEL_AMBIENT, globalAmbient);
      myGL.glLightModelf(GL.GL_LIGHT_MODEL_LOCAL_VIEWER, 1.0f);

      myGL.glEnable(GL.GL_LIGHT0);
      myGL.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, light0Position);
      myGL.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, lightAmbient);
      myGL.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, light0Position);
      myGL.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, lightAmbient);
      myGL.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, lightDiffuse);
      myGL.glLightfv(GL.GL_LIGHT0, GL.GL_SPECULAR, lightSpecular);

      myGL.glEnable(GL.GL_LIGHT1);
      myGL.glLightfv(GL.GL_LIGHT1, GL.GL_POSITION, light1Position);
      myGL.glLightfv(GL.GL_LIGHT1, GL.GL_AMBIENT, lightAmbient);
      myGL.glLightfv(GL.GL_LIGHT1, GL.GL_POSITION, light0Position);
      myGL.glLightfv(GL.GL_LIGHT1, GL.GL_AMBIENT, lightAmbient);
      myGL.glLightfv(GL.GL_LIGHT1, GL.GL_DIFFUSE, lightDiffuse);
      myGL.glLightfv(GL.GL_LIGHT1, GL.GL_SPECULAR, lightSpecular);

      //glLightfv(GL_LIGHT0, GL_DIFFUSE,lightDiffuse) ;
      //glLightfv(GL_LIGHT0, GL_SPECULAR,lightSpecular) ;

      //myGL.glRotatef(85.0f, 1.0f, 1.0f, 1.0f);
      myGL.glTranslated(transX, transY, 0.0);
      myGL.glRotated(rotateAngleX, 1.0, 0.0, 0.0);
      myGL.glRotated(rotateAngleY, 0.0, 1.0, 0.0);


myGL.glDisable(GL.GL_LIGHTING);
      if (axisVisible) {
        myGL.glColor3f(1.0f, 0.0f, 0.0f);
        myGL.glBegin(GL.GL_LINE_STRIP);
        myGL.glVertex3d(0.0, 0.0, 0.0);
        myGL.glVertex3d(1.0, 0.0, 0.0);
        myGL.glEnd();
        myGL.glColor3f(0.0f, 1.0f, 0.0f);
        myGL.glBegin(GL.GL_LINE_STRIP);
        myGL.glVertex3d(0.0, 0.0, 0.0);
        myGL.glVertex3d(0.0, 1.0, 0.0);
        myGL.glEnd();
        myGL.glColor3f(0.0f, 0.0f, 1.0f);
        myGL.glBegin(GL.GL_LINE_STRIP);
        myGL.glVertex3d(0.0, 0.0, 0.0);
        myGL.glVertex3d(0.0, 0.0, 1.0);
        myGL.glEnd();
        myGL.glColor3f(0.0f, 0.0f, 0.0f);
      }


      myGL.glEnable(GL.GL_LIGHTING);


      myGL.glDisable(GL.GL_LIGHTING);
      drawCurve();
      myGL.glEnable(GL.GL_LIGHTING);

      drawSurface();

      myGL.glPopMatrix();
      myGL.glFlush();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void drawCurve() {
    Curve c = null;
    float colorComp[] = new float[6];
    for (int i = 0; i < curves.size(); i++) {
      c = (Curve) curves.get(i);
      if( c.isRemoved() ){
        curves.remove(i);
        i--;
        continue;
      }

      if( !c.isVisible() ) continue;
      Point3d p = null;
      c.color.getComponents(colorComp);
      Color complimentColor1 = new Color(1-colorComp[0],colorComp[1],colorComp[2]);
      Color complimentColor2 = new Color(colorComp[0],1-colorComp[1],colorComp[2]);
      float[] compColor1 = new float[6];
      float[] compColor2 = new float[6];
      complimentColor1.getColorComponents(compColor1);
      complimentColor2.getColorComponents(compColor2);

      Color complimentColor3 = new Color(colorComp[0],colorComp[1],1-colorComp[2]);
      float[] compColor3 = new float[6];
      complimentColor3.getColorComponents(compColor3);

      myGL.glColor3f(compColor3[0],compColor3[1],compColor3[2]);



      if(c.drawTangent){
        Point3d vec = c.return_tangent(c.t);

        Point3d start = c.calculateAt(c.t);
        c.p = start;
        myGL.glBegin(GL.GL_LINES);
        myGL.glVertex3d(start.x,start.y,start.z);
        myGL.glVertex3d(start.x+ vec.x,start.y+vec.y,start.z+vec.z);
        myGL.glEnd();
      }

      if(c.showp){
        System.out.println("Inside c.showp in Viewer");
        myGL.glPointSize(5.0f);
        myGL.glBegin(GL.GL_POINTS);
        myGL.glVertex3d(c.p.getx(),c.p.gety(),c.p.getz());
        myGL.glEnd();
      }

      if(c.showControlPts){
        //show the controlPts
        //myGL.glColor3f(0.0f,0.0f,0.0f);
        myGL.glPointSize(5.0f);
        myGL.glBegin(GL.GL_POINTS);

        for(int j=0;j<c.ctrPt.size();j++){
          if(j%2 == 0) myGL.glColor3f(compColor1[0],compColor1[1],compColor1[2]);
          else  myGL.glColor3f(compColor2[0],compColor2[1],compColor2[2]);

          p = (Point3d) c.ctrPt.get(j);
          myGL.glVertex3d(p.getx(),p.gety(),p.getz());
        }
        myGL.glEnd();
        Point3d q = null;
        myGL.glColor3f(compColor1[0],compColor1[1],compColor1[2]);
        myGL.glBegin(GL.GL_LINES);
        for(int j=0;j<c.ctrPt.size()-1;j++){
          p = (Point3d) c.ctrPt.get(j);
          q = (Point3d) c.ctrPt.get(j+1);
          myGL.glVertex3d(p.getx(),p.gety(),p.getz());
          myGL.glVertex3d(q.getx(),q.gety(),q.getz());
          //myGL.glVertex3d(p.getx(),p.gety(),p.getz());
        }
          myGL.glEnd();
      }
      Point3d pts[] = c.generateRenderingPts();

      myGL.glColor3f(colorComp[0], colorComp[1], colorComp[2]);
      myGL.glBegin(GL.GL_LINE_STRIP);

      for (int j = 0; j < pts.length; j++) {
        p = pts[j];
        myGL.glVertex3d(p.getx(), p.gety(), p.getz());
      }
      myGL.glEnd();
    }
  }

  public void drawSurface() {
    Surface s = null;
    float colorComp[] = new float[6];

    for (int i = 0; i < surfaces.size(); i++) {
      s = (Surface) surfaces.get(i);
      if( s.isRemoved() ){
        surfaces.remove(i);
        i--;
        continue;
      }
      s.color.getComponents(colorComp);
          Color complimentColor1 = new Color(1-colorComp[0],colorComp[1],colorComp[2]);
              Color complimentColor2 = new Color(colorComp[0],1-colorComp[1],colorComp[2]);
              float[] compColor1 = new float[6];
              float[] compColor2 = new float[6];
              complimentColor1.getColorComponents(compColor1);
              complimentColor2.getColorComponents(compColor2);


      if( !s.isVisible() ) continue;

      Color complimentColor3 = new Color(colorComp[0],colorComp[1],1-colorComp[2]);
      float[] compColor3 = new float[6];
      complimentColor3.getColorComponents(compColor3);

      myGL.glColor3f(compColor3[0],compColor3[1],compColor3[2]);
      myGL.glDisable(GL.GL_LIGHTING);
      if(s.drawTangent){
        Vector vec = s.tangentPlane(s.u,s.v);
        Point3d fu = (Point3d)vec.get(0);
        Point3d fv = (Point3d) vec.get(1);

        Point3d start = s.calculateAt(s.u,s.v);
        s.p = start;
        myGL.glColor3f(compColor3[0],compColor3[1],compColor3[2]);
        myGL.glBegin(GL.GL_QUADS);
        myGL.glVertex3d(start.x+fu.x,start.y+fu.y,start.z+fu.z);
        myGL.glVertex3d(start.x+fv.x,start.y+fv.y,start.z+fv.z);
        myGL.glVertex3d(start.x-fu.x,start.y-fu.y,start.z-fu.z);
        myGL.glVertex3d(start.x- fv.x,start.y-fv.y,start.z-fv.z);
        myGL.glEnd();
      }


      if(s.showp){
        //System.out.println("Inside s.showp in Viewer");
        myGL.glColor3f(compColor2[0],compColor2[1],compColor2[2]);
        myGL.glPointSize(5.0f);
        myGL.glBegin(GL.GL_POINTS);
        //System.out.println(s.p);
        myGL.glVertex3d(s.p.getx(),s.p.gety(),s.p.getz());
        myGL.glEnd();
      }

      if(s.showIsoLines){
        myGL.glColor3f(compColor1[0],compColor1[1],compColor1[2]);
        if(s.whichLines == 0){
            Point3d pt[] = s.generate_u();

            myGL.glPointSize(5.0f);
            myGL.glBegin(GL.GL_POINTS);

            for (int j = 0; j < pt.length; j++) {
              myGL.glVertex3d(pt[j].getx(), pt[j].gety(), pt[j].getz());
            }
            myGL.glEnd();

          }
          else if (s.whichLines == 1){
            Point3d pt[] = s.generate_v();
            Point3d p = null;
            Point3d q = null;
            myGL.glPointSize(5.0f);
            myGL.glBegin(GL.GL_POINTS);
            for (int j = 0; j < pt.length; j++) {
              myGL.glVertex3d(pt[j].getx(), pt[j].gety(), pt[j].getz());
            }
            myGL.glEnd();
          }
        }
      myGL.glEnable(GL.GL_LIGHTING);

        if(s.showControlPts){
          myGL.glDisable(GL.GL_LIGHTING);
          Point3d p = null;

          myGL.glPointSize(5.0f);
          myGL.glBegin(GL.GL_POINTS);

          int size1 = s.bBox.size(),size2;
          for(int j=0;j<size1;j++){
            Object[] pts = ((Vector) s.bBox.get(j)).toArray();
            for(int k=0;k<pts.length;k++){
              p = (Point3d) pts[k];
              if(k%2 == 0) myGL.glColor3f(compColor1[0],compColor1[1],compColor1[2]);
              else  myGL.glColor3f(compColor2[0],compColor2[1],compColor2[2]);
              myGL.glVertex3d(p.getx(),p.gety(),p.getz());
            }
            //size2 = (Point3d)s.bBox[j].toArray()
          }
          myGL.glEnd();
          Point3d q = null;
          myGL.glBegin(GL.GL_LINES);
          for(int j=0;j<size1;j++){
            Object[] pts = ((Vector) s.bBox.get(j)).toArray();
            for(int k=0;k<pts.length-1;k++){
              p = (Point3d) pts[k];
              q = (Point3d) pts[k+1];
              myGL.glColor3f(compColor1[0],compColor1[1],compColor1[2]);

              myGL.glVertex3d(p.getx(),p.gety(),p.getz());
              myGL.glVertex3d(q.getx(),q.gety(),q.getz());
            }
            //size2 = (Point3d)s.bBox[j].toArray()
          }
          myGL.glEnd();
          myGL.glColor3f(compColor1[0],compColor1[1],compColor1[2]);
          Object[] pts = ((Vector) s.bBox.get(0)).toArray();
          myGL.glBegin(GL.GL_LINES);
          for(int j=0;j<pts.length;j++){
            for (int k = 0; k < size1 - 1; k++) {
              Object[] pts1 = ( (Vector) s.bBox.get(k)).toArray();
              p = (Point3d) pts1[j];
              Object[] pts2 = ( (Vector) s.bBox.get(k + 1)).toArray();
              q = (Point3d) pts2[j];

              myGL.glVertex3d(p.getx(), p.gety(), p.getz());
              myGL.glVertex3d(q.getx(), q.gety(), q.getz());
            }
          }
          myGL.glEnd();

          myGL.glEnable(GL.GL_LIGHTING);
        }

        Triangle3d triangles[] = s.generateRenderingTriangles();

        //myGL.glColor3f(colorComp[0], colorComp[1], colorComp[2]);
        float[] materialPlaneSpecular = {
            1.0f, 1.0f, 1.0f, 0.5f};
        float[] materialPlaneShininess = {
            16.0f};
        //float[] materialPlaneDiffuse   = {0.0f, 0.2f, 0.7f, 0.5f} ;
        float[] materialPlaneDiffuse = {
            colorComp[0], colorComp[1], colorComp[2], 1.0f}; //{0.0f, 0.2f, 0.7f, 0.5f} ;
        float[] materialPlaneAmbient = {
            0.0f, 0.0f, 0.2f, 1.0f};

        myGL.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, materialPlaneSpecular);
        myGL.glMaterialfv(GL.GL_FRONT, GL.GL_SHININESS, materialPlaneShininess);
        myGL.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, materialPlaneDiffuse);
        myGL.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, materialPlaneAmbient);

        Triangle3d tri = null;

        if( s.filltriangles ){
          myGL.glPolygonMode(GL.GL_FRONT_AND_BACK,GL.GL_FILL);
        }else{
          myGL.glPolygonMode(GL.GL_FRONT_AND_BACK,GL.GL_LINE);
        }
        myGL.glBegin(GL.GL_TRIANGLES);
        for (int j = 0; j < triangles.length; j++) {
          tri = triangles[j];
          myGL.glVertex3d(tri.p1.x, tri.p1.y, tri.p1.z);
          myGL.glVertex3d(tri.p2.x, tri.p2.y, tri.p2.z);
          myGL.glVertex3d(tri.p3.x, tri.p3.y, tri.p3.z);
        }
        myGL.glEnd();

        /*        myGL.glColor3d(0.0,0.0,0.0);
             myGL.glBegin(GL.GL_LINE_STRIP);
              for (int j=0;j<triangles.length;j++){
                tri = triangles[j];
                myGL.glVertex3d(tri.p1.x,tri.p1.y,tri.p1.z);
                myGL.glVertex3d(tri.p2.x,tri.p2.y,tri.p2.z);
                myGL.glVertex3d(tri.p3.x,tri.p3.y,tri.p3.z);
              }
              myGL.glEnd();
       */
    }

  }

  private void myInit() {
    myGL.glClearColor(1.0f, 1.0f, 1.0f, 0.0f);
  }

  public void myReshape(int w, int h) {
    winWidth = w;
    winHeight = h;
    myGL.glViewport(0, 0, w, h);
    myGL.glMatrixMode(GL.GL_PROJECTION);
    myGL.glLoadIdentity();


    if (w <= h) {
      myGL.glOrtho( -range, range,
                   -range * (float) h / (float) w,
                   range * (float) h / (float) w,
                   -range, range);

    }
    else {
      myGL.glOrtho( -range * (float) w / (float) h,
                   range * (float) w / (float) h,
                   -range, range,
                   -range, range);

    }
    myGL.glMatrixMode(GL.GL_MODELVIEW);
    myGL.glLoadIdentity();
  }

  public void keyboard(char key, int x, int y) {
    switch (key) {
      case 27:
        System.exit(0);
      default:
        break;
    }
  }

  private static final int BUFSIZE = 512;
  private boolean cullface = false;
  private boolean filltriangles = true;

  public void mouse(int button, int state, int x, int y) {
    /*if (button == GLUT.GLUT_MIDDLE_BUTTON && state == GLUT.GLUT_DOWN) {
      curX = x;
      curY = y;
      rotateState = true;
    }
    else if (button == GLUT.GLUT_MIDDLE_BUTTON && state == GLUT.GLUT_UP) {
      rotateState = false;
    }*/
   double X=0.0f,Y=0.0f;
    if (button == GLUT.GLUT_LEFT_BUTTON && state == GLUT.GLUT_DOWN) {
      curX = x;
      curY = y;
      translateState = true;
      rotateState = true;
    }
    else if (button == GLUT.GLUT_LEFT_BUTTON && state == GLUT.GLUT_UP) {
      translateState = false;
      rotateState = false;
    }
    if (button == GLUT.GLUT_RIGHT_BUTTON && state == GLUT.GLUT_DOWN) {
      int selectBuf[] = new int[BUFSIZE];
      int hits;
      int viewport[] = new int[4];
      System.out.println("X = "+x+"; Y = "+y);
      /*if (width <= height) {
        X = x/(4.0f*range);
        Y =  (float) height * y/ (4.0f*range*(float) width);

      myGL.glOrtho( -range, range,
                   -range * (float) h / (float) w,
                   range * (float) h / (float) w,
                   -range, range);

    }
    else {
      Y = y/(4.0f*range);
      X =  (float) width * x/ (4.0f*range*(float) height);



      myGL.glOrtho( -range * (float) w / (float) h,
                   range * (float) w / (float) h,
                   -range, range,
                   -range, range);

    }*/


    if(winWidth > winHeight){
      X = (2.0f * range * (float)x / (float) winHeight) -
          range * (float) winWidth / (float) winHeight; ;
      Y = - (2.0f * range * (float)y / (float) winHeight) + range;
    }
    else {
      X = (2.0f * range *(float)x) /(float)winWidth -  range;
      Y = (2.0f * range *(float)y)/(float)winWidth + (float)range * (float)winHeight / (float)winWidth;
    }

    //multiplying by inverse of rotation matrix

    double tempX = X*Math.cos(rotateAngleX) + Y*Math.sin(rotateAngleX);
    double tempY = Y*Math.cos(rotateAngleX) - X*Math.sin(rotateAngleX);

    X = tempX;
    Y = tempY;





    System.out.println("x = "+X+" ;y = "+Y);
    /*myGL.glGetIntegerv(GL.GL_VIEWPORT, viewport);

      myGL.glSelectBuffer(BUFSIZE, selectBuf);
      myGL.glPushMatrix();

      myGL.glRenderMode(GL.GL_SELECT);
      myGLU.gluPickMatrix( (double) x, (double) (viewport[3] - y), 5.0, 5.0,
                          viewport);
      myGL.glOrtho(0.0f, 8.0f, 0.0f, 8.0f, -0.5f, 2.5f);
      drawCurve();
      drawSurface();
      myGL.glPopMatrix();
      myGL.glFlush();

      hits = myGL.glRenderMode(GL.GL_RENDER);
      //processHits(hits, selectBuf);
     */
    }
  }

  public void processHits(int hits, int buffer[]) {
    int i, j;
    int names, ptr;

    System.out.println("hits = " + hits);
    ptr = 0;
    for (i = 0; i < hits; i++) { /*  for each hit  */
      names = buffer[ptr];
      System.out.println(" number of names for hit = " + names);
      ptr++;
      System.out.print("  z1 is " + buffer[ptr] + ";");
      ptr++;
      System.out.println(" z2 is " + buffer[ptr]);
      ptr++;
      System.out.print("   the name is ");
      for (j = 0; j < names; j++) { /*  for each name */
        System.out.print(buffer[ptr] + " ");
        ptr++;
      }
      System.out.println();
    }

  }

  public void mouseMotion(int x, int y) {
    if (translateState && myUT.glutGetModifiers() == myUT.GLUT_ACTIVE_CTRL) {
      int diffX = x - curX;
      int diffY = -y + curY;
      curX = x;
      curY = y;
      transX += diffX * 0.005;
      transY += diffY * 0.005;
      myUT.glutPostRedisplay();
    }else if (rotateState) {
      int diffX = x - curX;
      int diffY = -y + curY;
      curX = x;
      curY = y;
      rotateAngleY += diffX * rotateStep;
      rotateAngleX += diffY * rotateStep;
      myUT.glutPostRedisplay();
    }
  }

  /**
   * postRedisplay
   */
  public void postRedisplay() {
    myUT.glutPostRedisplay();
  }
  public boolean isCullface() {
    return cullface;
  }
  public void setCullface(boolean cullface) {
    this.cullface = cullface;
  }
  public boolean isFilltriangles() {
    return filltriangles;
  }
  public void setFilltriangles(boolean filltriangles) {
    this.filltriangles = filltriangles;
  }
}
